// See LICENSE for license details.
package sifive.freedom.unleashed.u500ml507devkit

import Chisel._
import chisel3.experimental.{withClockAndReset}

import freechips.rocketchip.config._
import freechips.rocketchip.diplomacy._

import sifive.blocks.devices.gpio._
import sifive.blocks.devices.pinctrl.{BasePin}

import sifive.fpgashells.shell.xilinx.ml507shell._

//-------------------------------------------------------------------------
// PinGen
//-------------------------------------------------------------------------

object PinGen {
  def apply(): BasePin = {
    new BasePin()
  }
}

//-------------------------------------------------------------------------
// U500ML507DevKitFPGAChip
//-------------------------------------------------------------------------

class U500ML507DevKitFPGAChip(implicit override val p: Parameters)
    extends ML507Shell
    with HasDebugJTAG {

  //-----------------------------------------------------------------------
  // DUT
  //-----------------------------------------------------------------------

  withClockAndReset(dut_clock, dut_reset) {
    val dut = Module(LazyModule(new U500ML507DevKitSystem).module)

    //---------------------------------------------------------------------
    // Connect peripherals
    //---------------------------------------------------------------------

    connectTerminal (dut)
    connectDDRMemory(dut)
    connectDebugJTAG(dut)
    connectSPI      (dut)
    connectUART     (dut)

    //---------------------------------------------------------------------
    // GPIO
    //---------------------------------------------------------------------

    val gpioParams = p(PeripheryGPIOKey)
    val gpio_pins = Wire(new GPIOPins(() => PinGen(), gpioParams(0)))

    GPIOPinsFromPort(gpio_pins, dut.gpio(0))

    gpio_pins.pins.zipWithIndex.foreach {
      case(pin, idx) =>
        pin.i.ival := dip(idx)
        led(idx) := pin.o.oval
    }
  }

}
